//============================================================================
// Name        : GPSDriver.cpp
// Author      : Reid Holmes
//
// TO COMPILE:
// g++ -o gps GPSDriver.cpp Model.cpp
// -or-
// clang++ -o gps GPSDriver.cpp Model.cpp
//============================================================================

#include <iostream>
#include <sstream>
#include <string>
#include <cassert>
#include "Model.h"
using namespace std;


int main() {

	Model model;

	// create some waypoints
	model.createWaypoint("Grad House", 43.469751, -80.540939);
	model.createWaypoint("DC", 43.472558, -80.542113);
	model.createWaypoint("Bomb Shelter", 43.471445, -80.544951);
	model.createWaypoint("RCH", 43.470277, -80.540735);

	// make sure we can get a real waypoint
	Waypoint *wpt = model.getWaypoint("DC");
	assert(wpt != NULL);

	wpt = model.getWaypoint("Garbage");
	assert(wpt == NULL);

	// create and populate a route
	const string EXAM_ROUTE = "Exam Prep";
	Route* route = model.createRoute(EXAM_ROUTE);
	assert(route != NULL);

	if (route != NULL) {
		route->appendWaypoint(model.getWaypoint("RCH"));
		route->appendWaypoint(model.getWaypoint("DC"));
		route->appendWaypoint(model.getWaypoint("invalid point"));
		route->appendWaypoint(model.getWaypoint("Bomb Shelter"));
		route->appendWaypoint(model.getWaypoint("Grad House"));
	}

	route = model.getRoute("Garbage route");
	assert(route == NULL);

	// try to create a duplicate route (should fail and return existing route)
	route = model.createRoute(EXAM_ROUTE);
	assert(route != NULL);

	const string ACTIVE_LOG = "Active Log";
	Track* track = model.createTrack(ACTIVE_LOG);
	assert(track != NULL);

	if (track != NULL) {
		track->appendPoint(new Point(43.469751, -80.540939));
		track->appendPoint(new Point(43.472558, -80.542113));
		track->appendPoint(new Point(43.471445, -80.544951));
		track->appendPoint(new Point(43.470277, -80.540735));
	}
	cout << "Track distance: " << track->distance() << " meters." << endl;

	cout << "GPS Driver Complete" << endl;

	return 0;
}
