/*
 * Model.cpp
 *
 *  Created on: 2012-10-04
 *      Author: rtholmes
 */

#include "Model.h"
#include <iostream>
#include <string>
#include <cmath>

using namespace std;

Model::Model() {
	// TODO Auto-generated constructor stub

}

Model::~Model() {
	cout << "Model::~Model()" << endl;
	// The model must delete its waypoints, routes, and tracks. Waypoints and tracks have to delete their own points.

	// delete routes
	for (map<string, Route*>::iterator routeIt = _routes.begin(); routeIt != _routes.end(); routeIt++) {
		delete routeIt->second;
	}

	// delete tracks
	for (map<string, Track*>::iterator trackIt = _tracks.begin(); trackIt != _tracks.end(); trackIt++) {
		delete trackIt->second;
	}

	// delete waypoints
	for (map<string, Waypoint*>::iterator wptIt = _waypoints.begin(); wptIt != _waypoints.end(); wptIt++) {
		delete wptIt->second;
	}
}

Point::Point(float lat, float lon) :
		_latitude(lat), _longitude(lon) {
}

Point::~Point() {
	cout << "\tPoint::~Point() - lat: " << _latitude << " lon: " << _longitude << endl;
}

Waypoint::~Waypoint() {
	cout << "Waypoint::~Waypoint() - name: " << _name << endl;
	delete _point;
}

Waypoint::Waypoint(const string name, const Point* point) :
		_name(name), _point(point) {
	cout << "Waypoint::Waypoint( " << name << ", point )" << endl;

}

Waypoint* Model::createWaypoint(string name, float lat, float lon) {

	Waypoint *waypoint = _waypoints[name];

	if (waypoint == NULL) {
		_waypoints[name] = new Waypoint(name, new Point(lat, lon));
		cout << "Model::createWaypoint( " << name << ", " << lat << ", " << lon << " ) - Waypoint created." << endl;
	} else {
		cout << "Model::createWaypoint( " << name << ", " << lat << ", " << lon << " ) - Waypoint already exists." << endl;
	}

	return _waypoints[name];
}

Route* Model::createRoute(string name) {
	Route *route = _routes[name];
	if (route == NULL) {
		_routes[name] = new Route(name);
		cout << "Model::createRoute( " << name << " ) - Route created." << endl;
	} else {
		cout << "Model::createRoute( " << name << " ) - Route already exists." << endl;
	}
	return _routes[name];
}

Track* Model::createTrack(string name) {
	Track *track = _tracks[name];
	if (track == NULL) {
		_tracks[name] = new Track(name);
		cout << "Model::createTrack( " << name << " ) - Track created." << endl;
	} else {
		cout << "Model::createTrack( " << name << " ) - Track already exists." << endl;
	}
	return _tracks[name];
}

Route::Route(string name) :
		_name(name) {
}
Route::~Route() {
	cout << "Route::~Route() - name: " << _name << endl;
}

Track::Track(string name) :
		_name(name) {
}

Track::~Track() {
	cout << "Track::~Track() - name: " << _name << endl;
	for (vector<const Point*>::const_iterator it = _points.begin(); it != _points.end(); it++) {
		const Point* p = (*it);
		delete p;
	}
}

void Track::appendPoint(const Point* p) {
	_points.push_back(p);
}

/*
 const Point* Track::getPoint(int index) const{
 return _points.at(index);
 }
 */

float Track::distance() const {
	float distance = 0;

	const Point* last = NULL;
	for (vector<const Point*>::const_iterator it = _points.begin(); it != _points.end(); it++) {
		const Point* p = (*it);

		if (last == NULL) {
			// on the first point we'll just compute the distance from a point to itself
			last = p;
		}

		distance += last->getDistance(p);
	}

	cout << "Track::distance() - distance for track: " << _name << " is: " << distance << endl;

	return distance;
}
float Point::getLatitude() const {
	return _latitude;
}

float Point::getLongitude() const {
	return _longitude;
}

float Point::getDistance(const Point* other) const {
	const float DEGREES_TO_RADIANS = (3.14159 / 180.0);

// Mean radius in KM
// google value (m): 6378137
	const float RADIUS_KM = 6378.137;

	float lat1 = DEGREES_TO_RADIANS * other->getLatitude();
	float long1 = DEGREES_TO_RADIANS * other->getLongitude();
	float lat2 = DEGREES_TO_RADIANS * getLatitude();
	float long2 = DEGREES_TO_RADIANS * getLongitude();

	float dlong = long2 - long1;
	float dlat = lat2 - lat1;

	float a = pow(sin(dlat / 2), 2) + (cos(lat1) * cos(lat2) * pow(sin(dlong / 2), 2));
	float c = 2 * atan2(sqrt(a), sqrt(1 - a));

	float dist = RADIUS_KM * c;

	return dist * 1000;
}

Route* Model::getRoute(string name) {
	Route* route = _routes[name];

	if (route == NULL) {
		cout << "Model::getRoute( " << name << " ) - Route not found." << endl;
	}

	return route;
}

ostream& operator<<(ostream& output, const Waypoint& wpt) {
	output << "Waypoint( )"; // << p.getName() << ", "<<p.getName()<<" )";
	return output;  // for multiple << operators.
}

string Waypoint::getName() const {
	return _name;
}

Waypoint* Model::getWaypoint(string name) {
	return _waypoints[name];
}

void Route::appendWaypoint(const Waypoint* wpt) {

	if (wpt != NULL) {
		_points.push_back(wpt);
	}
}

const Waypoint* Route::getWaypoint(int index) const {
	if (_points.size() < index) {
		return _points.at(index);
	}
	return NULL;
}

int Route::getSize() const {
	return _points.size();
}

string Route::getName() const {
	return _name;
}
