/*
 * Model.h
 *
 *  Created on: 2012-10-04
 *  Author: rtholmes
 */

#include <string>
#include <vector>
#include <map>
using namespace std;

#ifndef MODEL_H_
#define MODEL_H_

class Point {
public:
	Point(float lat, float lon);
	virtual ~Point();
	float getLatitude() const;
	float getLongitude() const;
	float getDistance(const Point* other) const;

private:
	const float _latitude;
	const float _longitude;
};

class Waypoint {
public:
	Waypoint(const string name, const Point* point);
	virtual ~Waypoint();
	string getName() const;
	Point getPoint() const;

private:
	const string _name;
	const Point* _point;
};

class Route {
public:
	Route(string name);
	virtual ~Route();

	string getName() const;
	int getSize() const;
	const Waypoint* getWaypoint(int index) const;

	void appendWaypoint(const Waypoint* wpt);

private:
	const string _name;
	vector<const Waypoint*> _points;
};

class Track {
public:
	Track(string name);
	virtual ~Track();

	const Point* getPoint(int index) const;
	int getSize() const;
	float distance() const;

	void appendPoint(const Point* p);

private:
	const string _name;
	vector<const Point*> _points;
};

class Model {
public:
	Model();
	virtual ~Model();

	// Creates and returns a waypoint. If a waypoint with that name already exists, return the already-existing one.
	Waypoint* createWaypoint(string name, float lat, float lon);

	// Creates and returns a route. If a route with that name already exists, return the already-existing one.
	Route* createRoute(string name);

	// Creates and returns a track. If a track with that name already exists, return the already-existing one.
	Track* createTrack(string name);

	// Gets the waypoint. Returns NULL if the waypoint doesn't exist.
	Waypoint* getWaypoint(string name);

	// Gets the route. Returns NULL if the route doesn't exist.
	Route* getRoute(string name);

	// Gets the track. Returns NULL if the track doesn't exist.
	Track* getTrack(string name);

private:
	map<string, Waypoint*> _waypoints;
	map<string, Track*> _tracks;
	map<string, Route*> _routes;
};

#endif /* MODEL_H_ */

